/*
 *
 * Copyright (C) 2009 Tomas Mazanec
 *
 * Author:   Tomas Mazanec <mazanec at utia.cas.cz>
 * Created:  Fri Aug 28 10:16:54 CEST 2009
 *
 * Application of CUDA in DSP algorithms 
 *	- Digital FIR filter implementation
 *
 */


// ///////////////////////////////////////////////////////////////////

#ifndef _CUTESTO_H_
#define _CUTESTO_H_

// ///////////////////////////////////////////////////////////////////
// ///////////////////////////////////////////////////////////////////

CUTBoolean readFileFCoeff(  const char *fpath, 
			    float *h_coeff,
			    const unsigned int M, 
			    CUTBoolean reverse)
{
    printf("Input file: %s\n", fpath);
    FILE *fi;

    unsigned int i=0;


    if ( (fi = fopen(fpath, "r")) == NULL) {printf("IO Error\n"); return(CUTFalse);}

    for (i=0; i<M; ++i)
    {
    
        // reverse order of coeffs
	if ( reverse == CUTTrue )
	{
    	    fscanf(fi, "%e\n", &h_coeff[M-(i+1)]);
	}
	else 
	{
	    fscanf(fi, "%e\n", &h_coeff[i]);
	}
	
	if ( (feof(fi) != 0) && (i!=(M-1)) )
	{
	    printf("File read Error.\n");
	    fclose(fi);
	    return(CUTFalse);
	}
    }
    
    fclose(fi);
    return(CUTTrue);

}

// ///////////////////////////////////////////////////////////////////

//	cutWriteFilef( "./data/input.dat", h_idata, x_len, 0.0)

CUTBoolean writeFileF(	const char *fpath,
			float *data,
			const unsigned int len)
{
    printf("Output file: %s\n", fpath);
    FILE *fo;

    unsigned int i=0;

    if ( (fo = fopen(fpath, "w")) == NULL) {printf("IO Error\n"); return(CUTFalse);}

    for (i=0; i<len; ++i)
    {
	if ( (fprintf(fo,"%.7e\n", data[i])) <= 0 )
	{
	    printf("File write Error.\n");
	    fclose(fo);
	    return(CUTFalse);
	}
    }

    fclose(fo);
    return(CUTTrue);
}

// ///////////////////////////////////////////////////////////////////

//	CUTBoolean res = cutComparef( reference, h_odata, y_len)

CUTBoolean compareF (   float *reference,
			float *data,
			const unsigned int len,
			const float epsilon)
{
    float diff;
    unsigned int i=0, err=0;
    bool result = true;
    
    for (i=0; i<len; ++i) 
    {
// Absolute
//	diff = reference[i] - data[i];
//	bool comp = (diff <= epsilon) && (diff >= -epsilon);
// Relative
	diff = (reference[i] - data[i] ) / reference[i];
	bool comp = (diff <= epsilon) && (diff >= -epsilon);
	if ( !comp ) {
	    err++;
#ifdef _DEBUG
	    printf("ErrNo.: %3d at %3d Ref:%.7e \tData:%.7e \trDiff:%.7e\n", err, i, reference[i], data[i], diff);
#endif
	}
	result &= comp;
    }
    if ( !result ) {printf("%d Errors of %d samples\n", err, len);}
    return (result) ? CUTTrue : CUTFalse;

}



// ///////////////////////////////////////////////////////////////////

#endif // _CUTESTO_H_

// ///////////////////////////////////////////////////////////////////
// ///////////////////////////////////////////////////////////////////
