/*
 *
 * Copyright (C) 2009 Tomas Mazanec
 *
 * Author:   Tomas Mazanec <mazanec at utia.cas.cz>
 * Created:  Fri Aug 28 10:16:54 CEST 2009
 *
 * Application of CUDA in DSP algorithms 
 *	- Digital FIR filter implementation
 *
 */

#include <stdio.h>

////////////////////////////////////////////////////////////////////////////////
// export C interface
extern "C" 
void computeGold( float* reference, float* coeff, float* idata, 
		const unsigned int M, const unsigned int x_len);
////////////////////////////////////////////////////////////////////////////////
//! Compute reference data set
//! Each element is multiplied with the number of threads / array length
//! @param reference  reference data, computed but preallocated
//! @param idata      input data as provided to device
//! @param len        number of elements in reference / idata
////////////////////////////////////////////////////////////////////////////////
void
computeGold( float* reference, float* coeff, float* idata,
	    const unsigned int M, const unsigned int x_len)
{
    printf("Y_len = N-M+1 !!!\n");
    unsigned int n = 0, k = 0;

#ifdef __DEVICE_EMULATION__

    double sum = 0;
    
    printf("SUPERGOLD, casting to DOUBLE according to emulator !\n");


    // reverse order of coeffs !
    for (n=0; n<x_len; n++)
    {
        sum = (double) 0.0;
        for ( k=0; k<M; k++)
	{
	    sum += (coeff[k] * idata[k+n]);
//	    printf("n=%d\t k=%d\n", n,k);
        }

	if (n<(x_len-M+1)) {reference[n] = (float)sum;}
    }

#else  

    float sum = 0;
    
    printf("SUPERGOLD, casting to FLOAT according to device !\n");

    // reverse order of coeffs !
    for (n=0; n<x_len; n++)
    {
        sum = (float) 0.0;
        for ( k=0; k<M; k++)
	{
	    sum += (coeff[k] * idata[k+n]);
//	    printf("n=%d\t k=%d\n", n,k);
        }

	if (n<(x_len-M+1)) {reference[n] = (float)sum;}
    }

#endif

}

