/*
 *
 * Copyright (C) 2009 Tomas Mazanec
 *
 * Author:   Tomas Mazanec <mazanec at utia.cas.cz>
 * Created:  Fri Aug 28 10:16:54 CEST 2009
 *
 * Application of CUDA in DSP algorithms 
 *	- Digital FIR filter implementation
 *
 */

#ifndef _TEMPLATE_KERNEL_H_
#define _TEMPLATE_KERNEL_H_

#include <stdio.h>

#define SDATA( index)      CUT_BANK_CHECKER(sdata, index)


#define MAXTHREADS 512

#define DOTHELAST

////////////////////////////////////////////////////////////////////////////////

//Input data texture reference // no conversion
// texture from linear memory - use tex1Dfetch() to fetch
texture<float, 1, cudaReadModeElementType> texIData;

//Coefficients texture reference // no conversion
// texture from cuda array - use tex1D() to fetch
texture<float, 1, cudaReadModeElementType> texCoeff;

// coeffs also in const mem.; assumed max. 4096 coeffs
__device__ __constant__ float c_coeff[4096];

/*
 g_odata = 1D array of input data of length x_len,
 M = FIR order, x_len = def. M*(K+1) input seq. length, 
 K = input seq. length multiplicator?, 
 L = No. of loops needed to enum. given FIR of order M, 
	i.e.: M>512 ? L=ceil(M/512) : L=1
*/

__global__ void
testKernel( float* g_odata, float* g_idata,
	    const unsigned int M, const unsigned int N, const unsigned int K,
	    const unsigned int L)
{

    // shared memory,the size is determined by the host application
    extern  __shared__  float sdata[];

    // c i o -> shmem
/*    float* odata = (float*) sdata;
    float* coeff = (float*) &odata[K*M];
    float* idata = (float*) &coeff[MAXTHREADS];
*/    // i o -> shmem
/*    float* odata = (float*) sdata;
    float* idata = (float*) &odata[K*M];
*/    // c o -> shmem
/*    float* odata = (float*) sdata;
    float* coeff = (float*) &odata[K*M];
*/  // o -> shmem
    float* odata = (float*) sdata;

    // thread index
    const unsigned int tid = threadIdx.x;
    // actual No. of threads
    const unsigned int NT = blockDim.x;
    // block index
    const unsigned int bid = blockIdx.x;
    // No. of blocks
    const unsigned int NB = gridDim.x;

    // to hold errors below epsilon
    double sum = 0;
    
    unsigned int i = 0, k = 0, l = 0, j = 0;


/*    coeff[tid] = (float)c_coeff[tid];
//    coeff[tid] = (float) tex1D( texCoeff, (unsigned int)tid );
    __syncthreads();
*/    
/*    for (k=0; k <= K; ++k) 
    {
	idata[tid+k*M] = (float) g_idata[(tid+k*M + bid*K*M)];
	//idata[tid+k*M] = (float) tex1Dfetch( texIData, (unsigned int)(tid+k*M + bid*K*M));
    }
    __syncthreads();
*/    
    for (k=0; k<K; ++k) 
    {
    for (j=0; j<L; ++j)
    {
    for (l=0; l<L; ++l)
    {

        for (i=0; i<NT; ++i) // M>512 ? i=0..511 : i=0..M-1 
        {
	    // casting to float is sufficient
	    // c i o -> shmem
/*	    sum += (float)(
		(float) coeff[i] *
		(float) idata[(i+tid+k*M)] );
*/
	    // i o -> shmem
/*	    sum += (float)(
		(float) tex1D( texCoeff, (unsigned int)i ) *
		(float) idata[(i+tid+k*M)] );
*/

	    // c o -> shmem
/*	    sum += (float)(
	    	(float) coeff[i] *
		(float) tex1Dfetch( texIData, (unsigned int)(i+tid+k*M + bid*K*M)) );
*/
	    // o -> shmem		
/*	    sum += (float)(
		(float) tex1D( texCoeff, (unsigned int)i ) *
		(float) tex1Dfetch( texIData, (unsigned int)(i+tid+k*M + bid*K*M)) );
*/		

	    sum += (float)(
		(float) tex1D( texCoeff, (unsigned int)(i+l*NT) ) *
		(float) tex1Dfetch( texIData, (unsigned int)(i+l*NT+j*NT+tid+k*M + bid*K*M)) );

		

        } // i-loop
    } // l-llop
        __syncthreads();
        odata[tid+j*NT+k*M] = (float)sum;
        __syncthreads();
        sum = (double)0.0;

    } // j-loop
    } // k-loop

#ifdef DOTHELAST
    // do the last sample to have y_len = x_len-M+1 samples
    // do it in last block
    if (bid == (NB-1)) 
    {
    if (tid == 0) 
    {
    for (j=0; j<1; ++j)
    {
    for (l=0; l<L; ++l)
    {
	for (i=0; i<NT; ++i)
        {
	    sum += (float)(
		(float) tex1D( texCoeff, (unsigned int)(i+l*NT) ) *
		(float) tex1Dfetch( texIData, (unsigned int)(i+l*NT+j*NT+tid+K*M + bid*K*M)) );
	}
    }}
    }}
    __syncthreads();

    // do the last sample;
    if (bid == (NB-1)) {
    if (tid == 0) {
	for (j=0; j<1; ++j)
	{
    	    odata[tid+j*NT+K*M] = (float)sum;
	}
    }}
    __syncthreads();
#endif

    for (k=0; k<K; ++k) 
    {
        for (j=0; j<L; ++j) 
	{
	    g_odata[tid+j*NT+k*M + bid*K*M] = (float)odata[tid+j*NT+k*M];
	}
    }

#ifdef DOTHELAST
    __syncthreads();

    // do the last sample;
    if (bid == (NB-1))
    {
    if (tid == 0)
    {
        for (j=0; j<1; ++j) 
	{
	    g_odata[tid+j*NT+K*M + bid*K*M] = (float)odata[tid+j*NT+K*M];
	}
    }}
    
#endif

}

#endif // #ifndef _TEMPLATE_KERNEL_H_
