/*
 *
 * Copyright (C) 2009 Tomas Mazanec
 *
 * Author:   Tomas Mazanec <mazanec at utia.cas.cz>
 * Created:  Fri Aug 28 10:16:54 CEST 2009
 *
 * Application of CUDA in DSP algorithms 
 *	- Digital FIR filter implementation
 *
 */

#ifndef _TEMPLATE_KERNEL_H_
#define _TEMPLATE_KERNEL_H_

#include <stdio.h>

#define SDATA( index)      CUT_BANK_CHECKER(sdata, index)


#define MAXTHREADS 512

////////////////////////////////////////////////////////////////////////////////

//Input data texture reference // no conversion
// texture from linear memory - use tex1Dfetch() to fetch
texture<float, 1, cudaReadModeElementType> texIData;

//Coefficients texture reference // no conversion
// texture from cuda array - use tex1D() to fetch
texture<float, 1, cudaReadModeElementType> texCoeff;


__global__ void
testKernel( float* g_odata,
	    const unsigned int M, const unsigned int N, const unsigned int K)
{

    // shared memory,the size is determined by the host application
    extern  __shared__  float odata[];

    // thread index
    const unsigned int tid = threadIdx.x;
    // actual No. of threads
    const unsigned int NT = blockDim.x;
    // block index
    const unsigned int bid = blockIdx.x;
    // No. of blocks
    const unsigned int NB = gridDim.x;



    // to hold errors below epsilon
    double sum = 0;
    unsigned int i = 0, k = 0; //, j = 0;

/*
    Block start
*/

    for (k=0; k<K; ++k) 
    {

        for (i=0; i<M; ++i)
        {
	    // casting to float is sufficient
	    sum += (float)(
		(float) tex1D( texCoeff, (unsigned int)i ) *
		(float) tex1Dfetch( texIData, (unsigned int)(i+tid+M*k + bid*K*M)) );
        }
        __syncthreads();
        odata[tid+M*k] = (float)sum;
        __syncthreads();
        sum = (double)0.0;
    }

    // do the last sample; no. of proper FIR outs = N-M+1
    if (tid == 0) 
    {
        for (i=0; i<M; ++i)
        {
	    sum += (float)(
		(float) tex1D( texCoeff, (unsigned int)i ) *
		(float) tex1Dfetch( texIData, (unsigned int)(i+tid+M*K + bid*K*M) )  );
        }
    }
    __syncthreads();
    if (tid == 0) 
	{ odata[tid+M*K] = (float)sum; }
    __syncthreads();
    
    for (k=0; k<K; ++k) 
    {
	g_odata[tid+k*M + bid*K*M] = (float)odata[tid+k*M];
    }
    __syncthreads();
    
    if (tid == 0)
    {
	g_odata[tid+K*M + bid*K*M] = (float)odata[tid+K*M];
    }

/*
    Block end
*/


}

#endif // #ifndef _TEMPLATE_KERNEL_H_
